"""
Originally posted in https://github.com/xonsh/xonsh/issues/2726#issuecomment-406447196 :
    Recently @CJ-Wright has started up a ``xonsh.lib`` (``xonsh.api``) sub-package, which is usable from pure Python.
    This is meant as a standard library for xonsh and downstream tools.
    Currently there are some xonsh-ish wrappers around ``os`` and ``subprocess``.
    We'd love to see more contributions to this effort! So if there is something
    like ``sh()`` that you'd like to see, by all means please help us add it!
"""
